
// disp.c - display for tube dac with TDA1541

#include <AT89X52.H>
#include <intrins.h>

// key scan codes
#define _OPTO_ 1
#define _RCA_ 2
#define _DISP_ 3

#define INOPT  1
#define INRCA  2


// pin alias for input/output signals
#define shiftclock P3_1
#define shiftd P3_2
#define setsource P1_0
#define mute P1_1
#define LED1 P1_2
#define LED2 P1_3
#define LED3 P1_4

// uma instrucao INC dura 12 us a 12 Mhz - 100instruct=1,200ms -> 1ms = 100x1000/1200=83,333...
#define _ONE_MS_DELAY_12MHZ_  83  // CALIBRAR COM OSCILOSCOPIO

// array to char pattern: 0     1     2     3     4     5     6     7     8     9   10 2. 11 4. 12 8.
unsigned char t[13] = { 0xFC, 0x60, 0xDA, 0xF2, 0x66, 0xB6, 0xBE, 0xE4, 0xFE, 0xF6, 0xDB, 0x67, 0xFF };
// Os sinais com ponto são os de indices 10, 11 e 12:  10->2.  11->4. 12->8.

unsigned char audsorce, key, keycount, displayon, repaint, audiostat;
unsigned kdelay;

void delay_ms(unsigned char ms_count){
    unsigned char ccount;
    while(ms_count){
        ccount=_ONE_MS_DELAY_12MHZ_;
        do{ ccount--; }while(ccount);
        ms_count--;
    }
}

void start4451(void){    // starts 4451 data
    shiftd = 1 ; // set data on
    shiftclock = 1;   // set clock on - pulse clock
    _nop_(); _nop_();
    shiftclock = 0 ; // set clock off
    shiftd = 0; // set data off
}

void end4451(void){    // ends 4451 data
    unsigned char i;
    shiftd = 0 ; // set data off
    for(i=0;i<3;i++){
        shiftclock = 1;   // set clock on - pulse clock
        _nop_(); _nop_();
        shiftclock = 0 ; // set clock off
    }
}

void show_digit(unsigned char pat){
    unsigned char i;
    for(i=0;i<=7;i++){
        if(pat&0x80)shiftd = 1;  // set data on
        else shiftd = 0 ; // set data off
        shiftclock = 1;   // set clock on - pulse clock
        pat <<= 1;  // one bit right
        shiftclock = 0 ; // set clock off
    }
    //shiftclock = 0 ; // set clock off
}

void clear_disp(void){
    unsigned char i;
    P3 = 0xF4;  // 11110100 set data on
    shiftclock = 1;  // set clock on - pulse clock
    _nop_(); _nop_();
    shiftclock = 0 ; // set clock off
    shiftd = 0; // set data off
    for(i=0;i<=35;i++){
        shiftclock = 1 ; // set clock on
        _nop_(); _nop_();
        shiftclock = 0;   // set clock off
    }
    shiftclock = 0 ; // set clock off
}

void keyboard(void){
    key=0;
    if(!P3_4)key=_OPTO_;
    if(!P3_5)key=_RCA_;
    if(!P3_6)key=_DISP_;
    if(key)keycount++;
    else keycount=0;
    if((keycount>50)&&(kdelay==0)){  // if key pressed
       if((key==_OPTO_)&&(audsorce!=INOPT)){ audsorce=INOPT; setsource=1; repaint=1; }  // opto
       if((key==_RCA_)&&(audsorce!=INRCA)){ audsorce=INRCA; setsource=0; repaint=1; }  //  rca
       if(key==_DISP_){  // display
           if(displayon){ displayon=0; clear_disp(); }
           else { displayon=1; repaint=1; }
       }
       keycount=0; kdelay=1000;
    }
    if(kdelay)kdelay--;
}

void display(void){
    unsigned char f;
    P3 = 0xF0;  // 11110000
    f = (P2 & 0x70);  // select bits F2 F4 F5 : F1=P2_4  F0=P2_5  F2=P2_6
    if(audiostat!=f)repaint=1;
    audiostat=f;
    if(!f)mute=1;
    else  mute=0;
    if(!displayon)return;
    if(!repaint)return;
    P1|=0x1C; // turn off leds
    start4451();
    if(audsorce==INOPT)show_digit(t[1]);  // optical input =1
    else if(audsorce==INRCA)show_digit(t[2]);  // rca input =2
    switch(f){
        case 0x10 :
        case 0x60 : show_digit(t[4]); show_digit(t[11]); show_digit(t[1]); LED1=0;  break; // 44.1 khz
        case 0x50 : show_digit(t[4]); show_digit(t[11]); show_digit(t[0]);  break; // 44.0 khz
        case 0x20 :
        case 0x40 : show_digit(t[4]); show_digit(t[12]); show_digit(t[0]); LED2=0; break; // 48.0 khz
        case 0x30 :
        case 0x70 : show_digit(t[3]); show_digit(t[10]); show_digit(t[0]);  break; // 32.0 khz
         default  : show_digit(t[0]); show_digit(t[0]);  show_digit(t[0]); LED3=0; break; // out of range
    }
    end4451();
    repaint=0;  // keeps display static until next change of inputs
}

void main(){
    P0 = 0;
    P1 = 0x03; // source opto, mute on
    P2 = 0x70;
    P3 = 0xF0;
    clear_disp();  audsorce=INRCA; displayon=1; repaint=1;  display();  // DEBUG - ELIMINAR
    delay_ms(100);
    P0 = 0;
    P1 = 0x03; // source opto, mute on
    P2 = 0x70;
    P3 = 0xF0;
    audsorce=INOPT;
    key=0; keycount=0; audiostat=0; kdelay=0; displayon=1; repaint=1;
    delay_ms(250);
    display();
    delay_ms(250); // ver aqui delay para manter mute ao ligar valvulas
    do{
        display();
        keyboard();
        delay_ms(1);
    }while(1);
}

// eof